#include <stdio.h>
#include <stdlib.h>
#include <assert.h>

#define MAX_SIZE 3

int mark_with_row(int size, int row, int* present, int* g);
int mark_with_column(int size, int col, int* present, int* g);
int mark_with_square(int size, int sq, int* present, int* g);

int check_rows(int size, int* g);
int check_columns(int size, int* g);
int check_squares(int size, int* g);


int* new_grid(int size, int* old)
{
   int i;
   int len = size*size*size*size;
   int* g = (int*) calloc( len, sizeof(int) );
   if (g == NULL) return NULL;

   if (old != NULL) {
      /* copy contents of old grid */
      for (i = 0; i < len; i++) {
         g[i] = old[i];
      }
   }

   return g;
}


int read_puzzle(int size, int* g, char* file_name)
{
   int i;
   int len = size*size*size*size;
   int* pg;

   FILE* fd = fopen(file_name, "r");

   if (fd == NULL) return 0;

   pg = g;
   for (i = 0; i < len; i++) {
      fscanf(fd, "%d", pg++);
   }

   return 1;
}


int print_puzzle(int size, int* g)
{
   int row, col;
   int len = size*size;

   int* pg = g;
   for (row = 0; row < len; row++) {
      for (col = 0; col < len; col++) {
         printf(" %d", *pg++);
      }
      printf("\n");
   }

   return 1;
}


int next_guess(int size, int cur, int id, int* g)
{
   int num;
   int present[1 + MAX_SIZE*MAX_SIZE];	/* extra space for "0" */

   int len = size*size;

   int row = id/len;
   int col = id%len;
   int  sq = col/size + size*(row/size);

   assert(size <= MAX_SIZE);   

   for (num = 0; num < len+1; num++) {
      present[num] = 0;
   }

   mark_with_row(size, row, present, g);
   mark_with_column(size, col, present, g);
   mark_with_square(size, sq, present, g);

   for (num = cur+1; num < len+1; num++) {
      if (present[num] == 0) return num;
   }

   return 0;
}


int mark_with_row(int size, int row, int* present, int* g)
{
   int col;
   int len = size*size;

   int* pg = g + row*len;
   for (col = 0; col < len; col++) {
     present[pg[col]] = 1;
   }

   return 1;
}


int mark_with_column(int size, int col, int* present, int* g)
{
   int row;
   int len = size*size;

   int* pg = g + col;
   for (row = 0; row < len; row++) {
     present[*pg] = 1;
     pg += len;
   }

   return 1;
}


int mark_with_square(int size, int sq, int* present, int* g)
{
   int i, row, col;
   int len = size*size;

   int row0 = size * (sq/size);
   int col0 = size * (sq%size);

   for (row = row0; row < row0+size; row++) {
      int* pg = g + col0 + row*len;
      for (col = col0; col < col0+size; col++) {
         present[*pg++] = 1;
      }
   }

   return 1;
}


int check_solution(int size, int* g)
{
   int ok;

   ok = check_rows(size, g);
   if (!ok) return 0;

   ok = check_columns(size, g);
   if (!ok) return 0;

   ok = check_squares(size, g);
   if (!ok) return 0;

   return 1;
}


int check_rows(int size, int* g)
{
   int row, col, num;
   int len = size*size;

   for (row = 0; row < len; row++) {
      int* pg = g + row*len;
      for (num = 1; num < len+1; num++) {
         int count = 0;
         for (col = 0; col < len; col++) {
            if (pg[col] == num) count++;
         }
         if (count != 1) {
            return 0;
	 }
      }
   }
   return 1;
}


int check_columns(int size, int* g)
{
   int  row, col, num;
   int  len = size*size;
   
   for (col = 0; col < len; col++) {
      for (num = 1; num < len+1; num++) {
         int* pg = g + col;
         int count = 0;
         for (row = 0; row < len; row++) {
            if (*pg == num) count++;
            pg += len;
         }
         if (count != 1) {
            return 0;
	 }
      }
   }
   return 1;
}


int check_squares(int size, int* g)
{
   int  row, col, sq, num;
   int  len = size*size;
   
   for (sq = 0; sq < len; sq++) {
      int row0 = size * (sq/size);
      int col0 = size * (sq%size);
      for (num = 1; num < len+1; num++) {
         int count = 0;
         for (row = row0; row < row0+size; row++) {
            int* pg = g + col0 + row*len;
            for (col = col0; col < col0+size; col++) {
               if (*pg++ == num) count++;
            }
         }
         if (count != 1) {
            return 0;
	 }
      }
   }
   return 1;
}
